@@ -691,6 +691,9 @@ ReorderIdent <- function(object, var, ...) {
 #' Linderman, G. C., Zhao, J., Kluger, Y. (2018). "Zero-preserving imputation
 #' of scRNA-seq data using low rank approximation." (bioRxiv:138677)
 #'
+#' @note RunALRA and associated functions are being moved to SeuratWrappers;
+#' for more information on SeuratWrappers, please see \url{https://github.com/satijalab/seurat-wrappers}
+#'
 #' @param object An object
 #' @param ... Arguments passed to other methods
 #'
@@ -718,6 +721,13 @@ ReorderIdent <- function(object, var, ...) {
 #' }
 #'
 RunALRA <- function(object, ...) {
+  .Deprecated(
+    new = 'SeruatWrappers::RunALRA',
+    msg = paste(
+      'RunALRA and associated functions are being moved to SeuratWrappers;',
+      'for more information on SeuratWrappers, please see https://github.com/satijalab/seurat-wrappers'
+    )
+  )
   UseMethod(generic = 'RunALRA', object = object)
 }
 
@@ -774,6 +784,11 @@ RunICA <- function(object, ...) {
 #'
 #' For details about stored LSI calculation parameters, see
 #' \code{PrintLSIParams}.
+#' 
+#' @note RunLSI is being moved to Signac. Equivalent functionality can be 
+#' achieved via the Signac::RunTFIDF and Signac::RunSVD functions; 
+#' for more information on Signac, please see
+#' \url{https://github.com/timoast/Signac}
 #'
 #' @param object Seurat object
 #' @param ... Arguments passed to other methods
@@ -782,6 +797,14 @@ RunICA <- function(object, ...) {
 #' @export RunLSI
 #'
 RunLSI <- function(object, ...) {
+  .Deprecated(
+    new = 'Signac::RunTFIDF',
+    msg = paste(
+      "RunLSI is being moved to Signac. Equivalent functionality can be", 
+      "achieved via the Signac::RunTFIDF and Signac::RunSVD functions; for",
+      "more information on Signac, please see https://github.com/timoast/Signac"
+    )
+  )
   UseMethod(generic = "RunLSI", object = object)
 }
 
