@@ -22,10 +22,13 @@ NULL
 #' @param assay Name of assay to use
 #' @param name Name for the expression programs
 #' @param seed Set a random seed
+#' @param search Search for symbol synonyms for features in \code{features} that
+#' don't match features in \code{object}? Searches the HGNC's gene names database;
+#' see \code{\link{UpdateSymbolList}} for more details
+#' @param ... Extra parameters passed to \code{\link{UpdateSymbolList}}
 #'
 #' @return Returns a Seurat object with module scores added to object meta data
 #'
-# @importFrom Hmisc cut2
 #' @importFrom ggplot2 cut_number
 #' @importFrom Matrix rowMeans colMeans
 #'
@@ -70,7 +73,9 @@ AddModuleScore <- function(
   k = FALSE,
   assay = NULL,
   name = 'Cluster',
-  seed = 1
+  seed = 1,
+  search = FALSE,
+  ...
 ) {
   set.seed(seed = seed)
   assay.old <- DefaultAssay(object = object)
@@ -92,6 +97,48 @@ AddModuleScore <- function(
     features <- lapply(
       X = features,
       FUN = function(x) {
+        missing.features <- setdiff(x = x, y = rownames(x = object))
+        if (length(x = missing.features) > 0) {
+          warning(
+            "The following features are not present in the object: ",
+            paste(missing.features, collapse = ", "),
+            ifelse(
+              test = search,
+              yes = ", attempting to find updated synonyms",
+              no = ", not searching for symbol synonyms"
+            ),
+            call. = FALSE,
+            immediate. = TRUE
+          )
+          if (search) {
+            tryCatch(
+              expr = {
+                updated.features <- UpdateSymbolList(symbols = missing.features, ...)
+                names(x = updated.features) <- missing.features
+                for (miss in names(x = updated.features)) {
+                  index <- which(x == miss)
+                  x[index] <- updated.features[miss]
+                }
+              },
+              error = function(...) {
+                warning(
+                  "Could not reach HGNC's gene names database",
+                  call. = FALSE,
+                  immediate. = TRUE
+                )
+              }
+            )
+            missing.features <- setdiff(x = x, y = rownames(x = object))
+            if (length(x = missing.features) > 0) {
+              warning(
+                "The following features are still not present in the object: ",
+                paste(missing.features, collapse = ", "),
+                call. = FALSE,
+                immediate. = TRUE
+              )
+            }
+          }
+        }
         return(intersect(x = x, y = rownames(x = object)))
       }
     )
@@ -383,7 +430,6 @@ CellCycleScoring <- function(
   set.ident = FALSE,
   ...
 ) {
-  CheckDots(..., fxns = 'AddModuleScore')
   name <- 'Cell Cycle'
   features <- list('S.Score' = s.features, 'G2M.Score' = g2m.features)
   object.cc <- AddModuleScore(
@@ -802,6 +848,128 @@ ExpVar <- function(x) {
   return(log1p(x = var(x = expm1(x = x))))
 }
 
+#' Get updated synonyms for gene symbols
+#'
+#' Find current gene symbols based on old or alias symbols using the gene
+#' names database from the HUGO Gene Nomenclature Committee (HGNC)
+#'
+#' @details For each symbol passed, we query the HGNC gene names database for
+#' current symbols that have the provided symbol as either an alias
+#' (\code{alias_symbol}) or old (\code{prev_symbol}) symbol. All other queries
+#' are \strong{not} supported.
+#'
+#' @note This function requires internet access
+#'
+#' @param symbols A vector of gene symbols
+#' @param timeout Time to wait before cancelling query in seconds
+#' @param several.ok Allow several current gene sybmols for each provided symbol
+#' @param verbose Show a progress bar depicting search progress
+#' @param ... Extra parameters passed to \code{\link[httr]{GET}}
+#'
+#' @return For \code{GeneSymbolThesarus}, if \code{several.ok}, a named list
+#' where each entry is the current symbol found for each symbol provided and the
+#' names are the provided symbols. Otherwise, a named vector with the same information.
+#'
+#' @source \url{https://www.genenames.org/} \url{http://rest.genenames.org/}
+#'
+#' @importFrom utils txtProgressBar setTxtProgressBar
+#' @importFrom httr GET accept_json timeout status_code content
+#'
+#' @rdname UpdateSymbolList
+#' @name UpdateSymbolList
+#'
+#' @export
+#'
+#' @seealso \code{\link[httr]{GET}}
+#'
+#' @examples
+#' \dontrun{
+#' GeneSybmolThesarus(symbols = c("FAM64A"))
+#' }
+#'
+GeneSymbolThesarus <- function(
+  symbols,
+  timeout = 10,
+  several.ok = FALSE,
+  verbose = TRUE,
+  ...
+) {
+  db.url <- 'http://rest.genenames.org/fetch'
+  search.types <- c('alias_symbol', 'prev_symbol')
+  synonyms <- vector(mode = 'list', length = length(x = symbols))
+  not.found <- vector(mode = 'logical', length = length(x = symbols))
+  multiple.found <- vector(mode = 'logical', length = length(x = symbols))
+  names(x = multiple.found) <- names(x = not.found) <- names(x = synonyms) <- symbols
+  if (verbose) {
+    pb <- txtProgressBar(max = length(x = symbols), style = 3, file = stderr())
+  }
+  for (symbol in symbols) {
+    sym.syn <- character()
+    for (type in search.types) {
+      response <- GET(
+        url = paste(db.url, type, symbol, sep = '/'),
+        config = c(accept_json(), timeout(seconds = timeout)),
+        ...
+      )
+      if (!identical(x = status_code(x = response), y = 200L)) {
+        next
+      }
+      response <- content(x = response)
+      if (response$response$numFound != 1) {
+        if (response$response$numFound > 1) {
+          warning(
+            "Multiple hits found for ",
+            symbol,
+            " as ",
+            type,
+            ", skipping",
+            call. = FALSE,
+            immediate. = TRUE
+          )
+        }
+        next
+      }
+      sym.syn <- c(sym.syn, response$response$docs[[1]]$symbol)
+    }
+    not.found[symbol] <- length(x = sym.syn) < 1
+    multiple.found[symbol] <- length(x = sym.syn) > 1
+    if (length(x = sym.syn) == 1 || (length(x = sym.syn) > 1 && several.ok)) {
+      synonyms[[symbol]] <- sym.syn
+    }
+    if (verbose) {
+      setTxtProgressBar(pb = pb, value = pb$getVal() + 1)
+    }
+  }
+  if (verbose) {
+    close(con = pb)
+  }
+  if (sum(not.found) > 0) {
+    warning(
+      "The following symbols had no synonyms: ",
+      paste(names(x = which(x = not.found)), collapse = ', '),
+      call. = FALSE,
+      immediate. = TRUE
+    )
+  }
+  if (sum(multiple.found) > 0) {
+    msg <- paste(
+      "The following symbols had multiple synonyms:",
+      paste(names(x = which(x = multiple.found)), sep = ', ')
+    )
+    if (several.ok) {
+      message(msg)
+      message("Including anyways")
+    } else {
+      warning(msg, call. = FALSE, immediate. = TRUE)
+    }
+  }
+  synonyms <- Filter(f = Negate(f = is.null), x = synonyms)
+  if (!several.ok) {
+    synonyms <- unlist(x = synonyms)
+  }
+  return(synonyms)
+}
+
 #' Calculate the variance to mean ratio of logged values
 #'
 #' Calculate the variance to mean ratio (VMR) in non-logspace (return answer in
@@ -938,6 +1106,35 @@ PercentageFeatureSet <- function(
   return(percent.featureset)
 }
 
+#' Regroup idents based on meta.data info
+#'
+#' For cells in each ident, set a new identity based on the most common value
+#' of a specified metadata column.
+#'
+#' @param object Seurat object
+#' @param metadata Name of metadata column
+#' @return A Seurat object with the active idents regrouped
+#'
+#' @export
+#'
+#' @examples
+#' pbmc_small <- RegroupIdents(pbmc_small, metadata = "groups")
+#'
+RegroupIdents <- function(object, metadata) {
+  for (ii in levels(x = object)) {
+    ident.cells <- WhichCells(object = object, idents = ii)
+    if (length(x = ident.cells) == 0) {
+      next
+    }
+    new.ident <- names(x = which.max(x = table(object[[metadata]][ident.cells, ])))
+    if (is.null(x = new.ident)) {
+      stop("Cluster ", ii, " contains only cells with NA values in the '", metadata, "' metadata column.")
+    }
+    Idents(object = object, cells = ident.cells) <- new.ident
+  }
+  return(object)
+}
+
 #' Stop Cellbrowser web server
 #'
 #' @importFrom reticulate py_module_available
@@ -959,6 +1156,52 @@ StopCellbrowser <- function() {
   }
 }
 
+#' @rdname UpdateSymbolList
+#'
+#' @return For \code{UpdateSymbolList}, \code{symbols} with updated symbols from
+#' HGNC's gene names database
+#'
+#' @export
+#'
+#' @examples
+#' \dontrun{
+#' UpdateSymbolList(symbols = cc.genes$s.genes)
+#' }
+#'
+UpdateSymbolList <- function(
+  symbols,
+  timeout = 10,
+  several.ok = FALSE,
+  verbose = TRUE,
+  ...
+) {
+  new.symbols <- suppressWarnings(expr = GeneSymbolThesarus(
+    symbols = symbols,
+    timeout = timeout,
+    several.ok = several.ok,
+    verbose = verbose,
+    ...
+  ))
+  if (length(x = new.symbols) < 1) {
+    warning("No updated symbols found", call. = FALSE, immediate. = TRUE)
+  } else {
+    if (verbose) {
+      message("Found updated symbols for ", length(x = new.symbols), " symbols")
+      x <- sapply(X = new.symbols, FUN = paste, collapse = ', ')
+      message(paste(names(x = x), x, sep = ' -> ', collapse = '\n'))
+    }
+    for (sym in names(x = new.symbols)) {
+      index <- which(x = symbols == sym)
+      symbols <- append(
+        x = symbols[-index],
+        values = new.symbols[[sym]],
+        after = index - 1
+      )
+    }
+  }
+  return(symbols)
+}
+
 #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 # Methods for Seurat-defined generics
 #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -1191,7 +1434,7 @@ CheckDots <- function(..., fxns = NULL) {
     )
     if (length(x = unused) > 0) {
       msg <- paste0(
-        "The following arguments are not used: ", 
+        "The following arguments are not used: ",
         paste(unused, collapse = ', ')
       )
       switch(
